#!/usr/bin/env python
# -*- coding: utf-8 -*-


"""
* Filename: tools.py
* Description:
* Time: 2025.02.05
* Author: wh0am1i
*/
"""

import json
import os
import time
import datetime
import hashlib
from zoomeye import config
from zoomeye.sdk import ZoomEyeDict
from zoomeye import show

zoomeye_dir = os.path.expanduser(config.ZOOMEYE_CONFIG_PATH)


def md5_convert(string):
    """
    calculate the md5 of a string
    :param string: input string
    :return: md5 string
    """
    m = hashlib.md5()
    m.update(string.encode('utf-8'))
    return m.hexdigest()


def convert_str(s):
    """
    convert arbitrary string to another string, for print and human readable
    :param:
    :return:
    """
    res = []
    d = {
        '\n': '\\n',
        '\r': '\\r',
        '\t': '\\t',
        '\b': '\\b',
        '\a': '\\a',
    }
    for c in s:
        if c in d.keys():
            res.append(d[c])
        else:
            res.append(c)
    return ''.join(res)


def omit_str(text, index=0):
    """
    ignore part of the string
    :param text: str, text to be omitted
    :param index: int, position omitted on the longest basis
    :return: str, ex: string...
    """
    if len(text) < config.STRING_MAX_LENGTH:
        return text

    return text[:config.STRING_MAX_LENGTH - index] + '...'


class Cache:
    """
    used to cache the data obtained from the api to the local,
    or directly clip the file from the local.
    """

    def __init__(self, dork, page, pagesize, facets, fields, sub_type):
        self.dork = dork
        self.page = page
        self.pagesize = pagesize
        self.facets = facets
        self.fields = fields
        self.sub_type = sub_type
        self.cache_folder = os.path.expanduser(config.ZOOMEYE_CACHE_PATH)
        cache_filename = "{}_{}_{}_{}".format(dork, facets, fields, sub_type)
        self.filename = self.cache_folder + config.FILE_NAME.format(md5_convert(cache_filename), page*pagesize)

    def check(self):
        """
        used to detect whether there is a local cache file,
        and check whether the local cache file expires.
        :return: bool
        """
        # exist?
        if not os.path.exists(self.filename):
            return False
        # check expired
        file_create_time = os.path.getatime(self.filename)
        # expired time is five day, time stamp = 60 * 60 * 24 * 5
        if int(time.time()) - int(file_create_time) > config.EXPIRED_TIME:
            # delete expired cache file
            os.remove(self.filename)
            return False
        # the cache file exists and has not expired
        else:
            return True

    def save(self, data):
        # folder is exists?
        if not os.path.isdir(self.cache_folder):
            try:
                # create folder when folder does not exists
                os.makedirs(self.cache_folder)
            except OSError:
                raise Exception("init dirs failed {}".format(self.cache_folder))
        # write to local
        with open(self.filename, 'w') as f:
            f.write(json.dumps(data))

    def load(self):
        """
        files in the cache from the local folder
        :return:
        """
        with open(self.filename, 'r') as f:
            read_data = f.read()
        data_json = json.loads(read_data)
        return data_json

    def update(self):
        pass


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
* Filename: cli.py
* Description: cli program entry
* Time: 2020.11.30
* Author: wh0am1i
*/
"""

import os
import sys
import argparse

module_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
sys.path.insert(1, module_path)

from zoomeye import core
from zoomeye.config import BANNER


def get_version():
    print(BANNER)


class ZoomEyeParser(argparse.ArgumentParser):
    def error(self, message):
        self.print_help()
        sys.exit(2)


def main():
    """
    parse user input args
    :return:
    """

    parser = ZoomEyeParser(prog='zoomeye')
    subparsers = parser.add_subparsers()
    # show ZoomEye-python version number
    parser.add_argument(
        "-v",
        "--version",
        action="store_true",
        help="show program's version number and exit"
    )

    # zoomeye account info
    parser_info = subparsers.add_parser("info", help="Show ZoomEye account info")
    parser_info.set_defaults(func=core.info)

    # initial account configuration related commands
    parser_init = subparsers.add_parser("init", help="Initialize the token for ZoomEye-python")
    parser_init.add_argument("-apikey", help="ZoomEye API Key", default=None, metavar='[api key]')
    parser_init.set_defaults(func=core.init)

    # query zoomeye data
    search_v2_parser = subparsers.add_parser("search",
                                             help="get network asset information based on query conditions.")
    search_v2_parser.add_argument("dork", type=str, help="search key word(eg=baidu.com)", default=None)

    search_v2_parser.add_argument(
        "-facets",
        default='',
        type=str,
        help=('''
                  if this parameter is specified,
                         the corresponding data will be displayed
                         at the end of the returned result.
                         supported : 'product', 'device', 'service', 'os', 'port', 'country', 'subdivisions', 'city'
              '''),
        metavar='facets'
    )
    search_v2_parser.add_argument(
        "-fields",
        default='',
        metavar='field=regexp',
        type=str,
        help=('''
                   display data based on input fields
                   please see: https://www.zoomeye.org/doc/
              ''')
    )
    search_v2_parser.add_argument("-sub_type", type=str, help="specify the type of data to search",
                                  choices=('v4', 'v6', 'web', 'all'), default='all')
    search_v2_parser.add_argument("-page", metavar='page', type=int, help="view the page of the query result",
                                  default=1)
    search_v2_parser.add_argument('-pagesize', metavar='pagesize', type=int,
                                  help="specify the number of pagesize to search", default=20)
    search_v2_parser.add_argument(
        "-figure",
        help="Pie chart or bar chart showing data，can only be used under facet and stat",
        choices=('pie', 'hist'),
        default=None
    )
    search_v2_parser.add_argument(
        "-save",
        help="Save search to local file",
        action='store_true',
    )
    search_v2_parser.add_argument(
        "-force",
        help=(
            """
            Ignore the local cache and force the data to be obtained from the API
            """
        ),
        action="store_true"
    )
    search_v2_parser.set_defaults(func=core.search)

    parser_clear = subparsers.add_parser("clear", help="Manually clear the cache and user information")
    parser_clear.add_argument(
        "-setting",
        help="clear user api key and access token",
        action="store_true"
    )
    parser_clear.add_argument(
        "-cache",
        help="clear local cache file",
        action="store_true"
    )
    parser_clear.set_defaults(func=core.clear_file)

    args = parser.parse_args()

    if args.version:
        get_version()
        exit(0)

    try:
        args.func(args)
    except AttributeError:
        parser.print_help()


if __name__ == '__main__':
    main()
